<?php

namespace Hoolah\Hoolah\Controller\Adminhtml\System\Config;

use \Exception;
use \Throwable;

use \Magento\Backend\App\Action;
use \Magento\Backend\App\Action\Context;
use \Magento\Framework\Controller\Result\JsonFactory;

use \Hoolah\Hoolah\Controller\Main as HoolahMain;
use \Hoolah\Hoolah\Helper\API as HoolahAPI;

use \Hoolah\Hoolah\Model\Config\Source\OperationMode as OperationMode;
    
class SendLogs extends Action
{
    public static function logsPath()
    {
        return [
            BP.'/var/log/system.log',
            BP.'/var/log/debug.log'
        ];
    }
    
    public static function getLines($filepath, $timeScope, $linesScope)
    {
        $timeScope = $timeScope * 24 * 60 * 60;
        $now = time();
        
        $handle = @HoolahMain::fopen($filepath, "r");
        if ($handle === false) return false;
        
        $start = 0;
        $end = HoolahMain::filesize($filepath);
        
        while ($end - $start > 1000000)
        {
            fseek($handle, (int) (($end + $start) / 2));
            
            $time = false;
            while (!$time && ($line = HoolahMain::fgets($handle, 4096)))
                $time = strtotime(substr($line, 1, strpos($line, ']') - 1));
            
            if (!$time || $now - $time <= $timeScope)
                $end = (int) (($end + $start) / 2);
            else
                $start = (int) (($end + $start) / 2);
        }
        
        $output = '';
        
        $get = false;
        fseek($handle, $start);
        while ($line = HoolahMain::fgets($handle, 4096))
        {
            $time = strtotime(substr($line, 1, strpos($line, ']') - 1));
            if ($time)
            {
                $get = $now - $time <= $timeScope;
                
                if ($get && $linesScope == 'hoolah')
                    $get = strpos($line, 'hoolah') !== false || strpos($line, '... was') !== false;
            }
            
            if ($get)
                $output .= $line;
        }
        
        HoolahMain::fclose($handle);
        return trim($output);
    }
    
    protected $resultJsonFactory;
    
    protected $searchCriteriaBuilder;
    protected $sortOrderBuilder;
    
    protected $quoteRepository;
    protected $orderRepository;
    protected $invoiceRepository;
    protected $transactionRepository;
    
    protected $hlogsRepository;

    protected $hdata;
    
    // protected
    protected function getAPI()
    {
        $api = null;
        $credentials = null;

        switch ($this->_request->getPost('mode'))
        {
            case OperationMode::MODE_PRODUCTION_SANDBOX:
                $credentials = [@trim($this->_request->getPost('merchant_id')), @trim($this->_request->getPost('merchant_secret_test')), HOOLAH_API_HOST_SANDBOX];
                break;
            
            case OperationMode::MODE_DEV:
                $credentials = [@trim($this->_request->getPost('dev_merchant_id')), @trim($this->_request->getPost('dev_merchant_secret')), HOOLAH_URL_DEV];
                break;
            
            case OperationMode::MODE_DEV_SANDBOX:
                $credentials = [@trim($this->_request->getPost('dev_merchant_id')), @trim($this->_request->getPost('dev_merchant_secret_test')), HOOLAH_URL_DEV_SANDBOX];
                break;
            
            default:
                $credentials = [@trim($this->_request->getPost('merchant_id')), @trim($this->_request->getPost('merchant_secret')), HOOLAH_API_HOST_SANDBOX];
                break;
        }

        if (!$credentials)
            throw new Exception('Unknown mode');
        
        if (empty($credentials[0]) || empty($credentials[1]) || empty($credentials[2]))
            throw new Exception('Merchant credentials are empty');
        
        $api = new HoolahAPI(
            $credentials[0],
            $credentials[1],
            $credentials[2],
            $this->hdata->getCache(),
            $this->hdata->getVersion()
        );

        return $api;
    }

    public function getLogsURL()
    {
        $logs_url = null;

        $relates = @$this->_request->getPost('logs_relates').' '.@$this->_request->getPost('logs_details');

        switch ($this->_request->getPost('mode'))
        {
            case OperationMode::MODE_PRODUCTION_SANDBOX:
                $logs_url = sprintf(HOOLAH_LOG_PROD_SANDBOX, trim($this->_request->getPost('merchant_cdn_id')), urlencode($relates));
                break;
            
            case OperationMode::MODE_DEV:
                $logs_url = sprintf(HOOLAH_LOG_DEV, trim($this->_request->getPost('merchant_cdn_id')), urlencode($relates));
                break;
            
            case OperationMode::MODE_DEV_SANDBOX:
                $logs_url = sprintf(HOOLAH_LOG_DEV_SANDBOX, trim($this->_request->getPost('merchant_cdn_id')), urlencode($relates));
                break;
            
            default:
                $logs_url = sprintf(HOOLAH_LOG_PROD, trim($this->_request->getPost('merchant_cdn_id')), urlencode($relates));
                break;
        }

        return $logs_url;
    }

    protected function getDBLogs()
    {
        $data = '';

        try
        {
            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('created_at', date('Y-m-d', time() - @$this->_request->getPost('logs_time_scope') * 60*60*24), 'gt')
                ->setSortOrders([
                    $this->sortOrderBuilder->setField('created_at')->setDirection('ASC')->create(),
                    $this->sortOrderBuilder->setField('sequence')->setDirection('ASC')->create()
                ])
                ->create();
            
            $hlogs = $this->hlogsRepository->getList($searchCriteria);
            if ($hlogs->getItems())
            {
                $items = $hlogs->getItems();
                
                $data .= "Count: ".count($items)."\n";
                foreach ($items as $hlog)
                {
                    $data .= $hlog->getCreatedAt().' ['.$hlog->getIp().'/'.$hlog->getThread().']: '.$hlog->getDescription()."\n";
                    
                    if ($hlog->getDetails())
                        $data .= $hlog->getDetails()."\n";
                }
            }
            else
                $data .= "<empty>\n";
        }
        catch (\Throwable $e)
        {
            $data .= "Exception: ".$e->getMessage();
        }

        return $data;
    }

    protected function getQuotesLogs()
    {
        $data = '';

        try
        {
            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('hoolah_order_context_token', null, 'notnull')
                ->addFilter('updated_at', date('Y-m-d', time() - @$this->_request->getPost('logs_time_scope') * 60*60*24), 'gt')
                ->create();
            $quotes = $this->quoteRepository->getList($searchCriteria);
            if ($quotes->getTotalCount())
            {
                $data .= "Count: ".$quotes->getTotalCount()."\n";
                foreach ($quotes->getItems() as $quote)
                    $data .= json_encode($quote->toArray(), JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE)."\n";
            }
            else
                $data .= "<empty>\n";
        }
        catch (\Throwable $e)
        {
            $data .= "Exception: ".$e->getMessage();
        }

        return $data;
    }

    protected function getOrdersLogs()
    {
        $data = '';

        try
        {
            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('hoolah_order_context_token', null, 'notnull')
                ->addFilter('updated_at', date('Y-m-d', time() - @$this->_request->getPost('logs_time_scope') * 60*60*24), 'gt')
                ->create();
            $orders = $this->orderRepository->getList($searchCriteria);
            if ($orders->getTotalCount())
            {
                $data .= "Count: ".$orders->getTotalCount()."\n";
                foreach ($orders->getItems() as $order)
                    $data .= json_encode($order->toArray(), JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE)."\n";
            }
            else
                $data .= "<empty>\n";
        }
        catch (\Throwable $e)
        {
            $data .= "Exception: ".$e->getMessage();
        }

        return $data;
    }

    protected function getInvoicesLogs()
    {
        $data = '';

        try
        {
            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('transaction_id', 'hoolah_%', 'like')
                ->addFilter('updated_at', date('Y-m-d', time() - @$this->_request->getPost('logs_time_scope') * 60*60*24), 'gt')
                ->create();
            $ivoices = $this->invoiceRepository->getList($searchCriteria);
            if ($ivoices->getTotalCount())
            {
                $data .= "Count: ".$ivoices->getTotalCount()."\n";
                foreach ($ivoices->getItems() as $ivoice)
                    $data .= json_encode($ivoice->toArray(), JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE)."\n";
            }
            else
                $data .= "<empty>\n";
        }
        catch (\Throwable $e)
        {
            $data .= "Exception: ".$e->getMessage();
        }

        return $data;
    }

    protected function getTransactionsLogs()
    {
        $data = '';

        try
        {
            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('txn_id', 'hoolah_%', 'like')
                ->addFilter('created_at', date('Y-m-d', time() - @$this->_request->getPost('logs_time_scope') * 60*60*24), 'gt')
                ->create();
            $transactions = $this->transactionRepository->getList($searchCriteria);
            if ($transactions->getTotalCount())
            {
                $data .= "Count: ".$transactions->getTotalCount()."\n";
                foreach ($transactions->getItems() as $transaction)
                    $data .= json_encode($transaction->toArray(), JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE)."\n";
            }
            else
                $data .= "<empty>\n";
        }
        catch (\Throwable $e)
        {
            $data .= "Exception: ".$e->getMessage();
        }

        return $data;
    }

    // public
    /**
     * @param Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\SortOrderBuilder $sortOrderBuilder,
        \Magento\Quote\Model\QuoteRepository $quoteRepository,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Sales\Api\InvoiceRepositoryInterface $invoiceRepository,
        \Magento\Sales\Api\TransactionRepositoryInterface $transactionRepository,
        \Hoolah\Hoolah\Model\HoolahLogRepository $hlogsRepository,
        \Hoolah\Hoolah\Helper\Data $hdata
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;
        
        $this->quoteRepository = $quoteRepository;
        $this->orderRepository = $orderRepository;
        $this->invoiceRepository = $invoiceRepository;
        $this->transactionRepository = $transactionRepository;
        
        $this->hlogsRepository = $hlogsRepository;

        $this->hdata = $hdata;

        parent::__construct($context);
    }
    
    /**
     * Collect relations data
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        HoolahMain::load_configs();
        
        $result = [
            'ok' => false,
            'error' => [
                'response' => [
                    'code' => 500
                ],
                'body' => 'Unknown error'
            ]
        ];
        
        try
        {
            $api = $this->getAPI();
            $logs_url = $this->getLogsURL();
            
            if ($api)
            {
                // db logs
                $data = "DB logs\n------------------\n";
                $data .= $this->getDBLogs();
                
                $result['db_logs'] = $api->send_logs($data, $logs_url);
                if (!HoolahAPI::is_200($result['db_logs']))
                    $result['error'] = $result['db_logs'];
                $result['ok'] = HoolahAPI::is_200($result['db_logs']);
                
                // db data
                $data = "Quotes\n------------------\n";
                $data .= $this->getQuotesLogs();
                
                $data .= "Orders\n------------------\n";
                $data .= $this->getOrdersLogs();
                
                $data .= "Invoices\n------------------\n";
                $data .= $this->getInvoicesLogs();
                
                $data .= "Transactions\n------------------\n";
                $data .= $this->getTransactionsLogs();
                
                $result['db_data'] = $api->send_logs($data, $logs_url);
                if (!HoolahAPI::is_200($result['db_data']))
                    $result['error'] = $result['db_data'];
                $result['ok'] = $result['ok'] && HoolahAPI::is_200($result['db_data']);
    
                // file logs
                $data = "File logs\n------------------\n";

                foreach (self::logsPath() as $logPath)
                    $data .= self::getLines($logPath, @$this->_request->getPost('logs_time_scope'), @$this->_request->getPost('logs_lines_scope'));
                
                $result['file_logs'] = $api->send_logs($data, $logs_url);
                if (!HoolahAPI::is_200($result['file_logs']))
                    $result['error'] = $result['file_logs'];
                $result['ok'] = $result['ok'] && HoolahAPI::is_200($result['file_logs']);
                
                if ($result['ok'])
                    unset($result['error']);
            }
        }
        catch (\Throwable $e)
        {
            $result['ok'] = false;
            $result['error'] = [
                'response' => [
                    'code' => 500
                ],
                'body' => $e->getMessage()
            ];
        }
        
        $jsonf = $this->resultJsonFactory->create();
        
        return $jsonf->setData($result);
    }
}
