<?php

namespace Hoolah\Hoolah\Controller\Adminhtml\System\Config;

use \Magento\Backend\App\Action;
use \Magento\Backend\App\Action\Context;
use \Magento\Framework\Controller\Result\JsonFactory;

use \Hoolah\Hoolah\Controller\Main as HoolahMain;
use \Hoolah\Hoolah\Helper\API as HoolahAPI;
    
class ValidateMerchant extends Action
{
    protected $resultJsonFactory;

    protected $hdata;
    
    /**
     * @param Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        \Hoolah\Hoolah\Helper\Data $hdata
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;

        $this->hdata = $hdata;
        
        parent::__construct($context);
    }
    
    /**
     * Collect relations data
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        HoolahMain::load_configs();
        
        $result = [
            'ok' => false
        ];
        
        $for_check = [
            'ok' => [
                @trim($this->_request->getPost('merchant_id')),
                @trim($this->_request->getPost('merchant_secret')),
                HOOLAH_API_HOST_PROD
            ],
            'ok_test' => [
                @trim($this->_request->getPost('merchant_id')),
                @trim($this->_request->getPost('merchant_secret_test')),
                HOOLAH_API_HOST_SANDBOX
            ]
        ];
        
        if (@trim($this->_request->getPost('dev_merchant_secret')))
            $for_check['dev_ok'] = [
                @trim($this->_request->getPost('dev_merchant_id')),
                @trim($this->_request->getPost('dev_merchant_secret')),
                HOOLAH_URL_DEV
            ];
        if (@trim($this->_request->getPost('dev_merchant_secret_test')))
            $for_check['dev_ok_test'] = [
                @trim($this->_request->getPost('dev_merchant_id')),
                @trim($this->_request->getPost('dev_merchant_secret_test')),
                HOOLAH_URL_DEV_SANDBOX
            ];
        
        foreach ($for_check as $key => $credentials)
        {
            try
            {
                if (empty($credentials[0]) || empty($credentials[1]) || empty($credentials[2]))
                    $result[$key] = false;
                else
                {
                    $api = new HoolahAPI(
                        $credentials[0],
                        $credentials[1],
                        $credentials[2],
                        $this->hdata->getCache(),
                        $this->hdata->getVersion()
                    );
                    $response = $api->merchant_auth_login();
                    $result[$key] = HoolahAPI::is_200($response);
                }
            }
            catch (\Throwable $e)
            {
                $result[$key] = false;
            }
        }
        
        if (@trim($this->_request->getPost('merchant_cdn_id')))
        {
            try
            {
                $result['ok_cdn'] = HoolahAPI::url_exists(sprintf(HOOLAH_WIDGET_URL_CUSTOM, trim($this->_request->getPost('merchant_cdn_id'))));
            }
            catch (\Throwable $e)
            {
                $result['ok_cdn'] = false;
            }
        }
        
        $jsonf = $this->resultJsonFactory->create();
        
        return $jsonf->setData($result);
    }
}
