<?php

namespace Hoolah\Hoolah\Helper;

class API
{
    // const
    const AUTH_TOKEN_OPTION_NAME = 'auth_tokens';

    // static
    public static function is_200($response)
    {
        return @$response['response']['code'] == 200;
    }
    public static function is_201($response)
    {
        return @$response['response']['code'] == 201;
    }
    public static function is_202($response)
    {
        return @$response['response']['code'] == 202;
    }
    public static function is_401($response)
    {
        return @$response['response']['code'] == 401;
    }
    
    public static function get_message($response)
    {
        $result = null;
        
        if (@$response['body']['message'])
            $result = $response['body']['message'];
        else if (@$response['body']['errorMessages'])
            $result = $response['body']['errorMessages'];
        
        if (is_array($result))
            $result = implode(', ', $result);
        
        return $result;
    }
    
    public static function get_client($avoid_magento_curl_adapter = false)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $productMetadata = $objectManager->get('Magento\Framework\App\ProductMetadataInterface');
        
        if (version_compare($productMetadata->getVersion(), '2.4.6', '>='))
            return new \Hoolah\Hoolah\Helper\HTTP\LaminasClient(); //m2.4.6
        else
        {
            if ($avoid_magento_curl_adapter)
            {   
                if (version_compare($productMetadata->getVersion(), '2.3', '>='))
                    return new \Hoolah\Hoolah\Helper\HTTP\ZendClientWOMCA23(); //m2.3
                else
                    return new \Hoolah\Hoolah\Helper\HTTP\ZendClientWOMCA22(); //m2.2
            }
            else
            {
                if (version_compare($productMetadata->getVersion(), '2.3', '>='))
                    return new \Magento\Framework\HTTP\ZendClient(); //m2.3
                else
                    return new \Hoolah\Hoolah\Helper\HTTP\ZendClient(); //m2.2
            }
        }
    }
    
    public static function url_exists($url)
    {
        $client = self::get_client();
        $client->setUri($url);
        $client->setMethod('HEAD');
        $result = $client->request();
        
        $result = [
            'response' => [
                'code' => $result->getStatus()
            ]
        ];
        
        //$result = wp_remote_head($url);
        
        return self::is_200($result);
    }
    
    public static function get_content($url)
    {
        $client = self::get_client();
        $client->setUri($url);
        $result = $client->request();
        
        $result = [
            'body' => $result->getBody(),
            'response' => [
                'code' => $result->getStatus()
            ],
        ];
        
        return $result;
    }
    
    // private
    private $fstr_url = null;
    private $fstr_merchantID = null;
    private $fstr_merchantSecret = null;
    private $fstr_version = null;
    
    private $fobj_cache = null;

    private $fstr_token = null;

    private function get_auth_tokens()
    {
        $result = json_decode($this->fobj_cache->load(self::class), true);

        if (!is_array($result))
            $result = [];

        $result = @$result[self::AUTH_TOKEN_OPTION_NAME];

        if (!is_array($result))
            $result = [];
        
        return $result;
    }

    private function update_auth_tokens($tokens)
    {
        if (!is_array($tokens))
            $tokens = [];
        
        $result = json_decode($this->fobj_cache->load(self::class), true);

        if (!is_array($result))
            $result = [];
        
        $result[self::AUTH_TOKEN_OPTION_NAME] = $tokens;
        
        $this->fobj_cache->save(
            json_encode($result),
            self::class,
            [],
            60*60*24*30
        );
    }

    private function get_auth_token($merchant_id, $secret)
    {
        return @$this->get_auth_tokens()[md5($merchant_id.'_'.$secret)];
    }

    private function update_auth_token($merchant_id, $secret, $token)
    {
        $tokens = $this->get_auth_tokens();

        $tokens[md5($merchant_id.'_'.$secret)] = $token;

        $this->update_auth_tokens($tokens);
    }
    
    private function _request($path, $data = null, $method = null)
    {
        $result = null;
        
        $client = self::get_client($method == 'PATCH');
        $client->setHeaders('Content-Type', 'application/json');
        $client->setHeaders('Accept', 'application/json');
        $client->setHeaders('SHOPBACK-PLUGIN-PLATFORM', 'MAGENTO');
        $client->setHeaders('SHOPBACK-PLUGIN-VERSION', $this->fstr_version);

        if ($this->fstr_token)
            $client->setHeaders('Authorization', 'Bearer '.$this->fstr_token);
        
        if ($data !== null)
        {
            if (!$method)
                $method = 'POST';
            
            $client->setRawData(json_encode($data));
        }
        else
        {
            if (!$method)
                $method = 'GET';
        }

        $client->setMethod($method);
        $client->setUri($this->fstr_url.$path);
        
        $result = $client->request();
        
        $result = [
            'response' => [
                'code' => $result->getStatus()
            ],
            'body' => json_decode($result->getBody(), true)
        ];
        
        return $result;
    }

    private function request($path, $data = null, $method = null)
    {
        $result = null;
        
        if ($this->fstr_token)
            $result = $this->_request($path, $data, $method);

        if (!$this->fstr_token || self::is_401($result))
        {
            $result = $this->merchant_auth_login();

            if (self::is_200($result))
                $result = $this->_request($path, $data, $method);
        }
        
        return $result;
    }
    
    // public
    public function __construct($merchant_id, $merchant_secret, $url, $cache, $version = null)
    {
        $this->fstr_url = $url;
        $this->fstr_merchantID = $merchant_id;
        $this->fstr_merchantSecret = $merchant_secret;
        $this->fstr_version = $version;

        $this->fobj_cache = $cache;

        $this->fstr_token = $this->get_auth_token($this->fstr_merchantID, $this->fstr_merchantSecret);
    }
    
    public function merchant_auth_login()
    {
        $result = $this->_request('/merchant/auth/login', [
            'username' => $this->fstr_merchantID,
            'password' => $this->fstr_merchantSecret
        ]);
        
        if (self::is_200($result))
        {
            $this->fstr_token = $result['body']['token'];

            $this->update_auth_token($this->fstr_merchantID, $this->fstr_merchantSecret, $this->fstr_token);
        }
        
        return $result;
    }
    
    public function merchant_order_get($uuid)
    {
        $result = $this->request('/merchant/order/'.$uuid);
        
        return $result;
    }
    
    public function merchant_order_initiate($order)
    {
        $result = $this->request('/merchant/order/initiate', $order);
        
        return $result;
    }
    
    public function merchant_order_full_refund($uuid, $description = null)
    {
        $result = $this->request('/merchant/order/'.$uuid.'/full-refund', [
            'description' => $description
        ]);
        
        return $result;
    }
    
    public function merchant_order_partial_refund($uuid, $amount, $description = null, $items = null)
    {
        $result = $this->request('/merchant/order/'.$uuid.'/partial-refund', [
            'amount' => $amount,
            'items' => $items,
            'description' => $description
        ]);
        
        return $result;
    }
    
    public function merchant_order_finalize($uuid, $quote_id, $order_id, $email)
    {
        $result = $this->request('/merchant/order/finalize/'.$uuid, [
            'cart_id' => $quote_id,
            'email' => $email,
            'created_at' => date('c'),
            'merchant_order_id' => $order_id,
            'status' => 'paid',
            'order_uuid' => $uuid
        ]);
        
        return $result;
    }

    public function merchant_order_patch_order_id($uuid, $order_id)
    {
        $result = $this->request('/merchant/order/'.$uuid, [
            'merchantOrderId' => $order_id,
        ], 'PATCH');
        
        return $result;
    }
    
    public function send_logs($data, $url)
    {
        $client = self::get_client();
        
        $client->setAuth($this->fstr_merchantID, $this->fstr_merchantSecret, 'basic');
        
        $client->setMethod('POST');
        $client->setUri($url);
        
        $offset = 0;
        $length = 30000000;
        while ($offset < strlen($data))
        {
            $client->setRawData(substr($data, $offset, $length));
            
            $result = $client->request();
            
            $result = [
                'response' => [
                    'code' => $result->getStatus()
                ],
                'body' => $result->getBody()
            ];
            
            if (!self::is_200($result))
                return $result;
            
            $offset += $length;
        }
        
        return $result;
    }
}
