<?php

namespace Hoolah\Hoolah\Helper;

use \Magento\Store\Model\ScopeInterface;

use \Hoolah\Hoolah\Controller\Main as HoolahMain;
use \Hoolah\Hoolah\Model\Config\Source\OperationMode as OperationMode;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    const MODULE_NAME = 'Hoolah_Hoolah';
    
    protected $moduleList;
    protected $extSettings;
    protected $cache;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Hoolah\Hoolah\Helper\ExtSettings $extSettings,
        \Magento\Framework\App\CacheInterface $cache
    )
    {
        parent::__construct($context);
        
        $this->moduleList = $moduleList;
        $this->extSettings = $extSettings;
        $this->cache = $cache;
    }

    public function getCache()
    {
        return $this->cache;
    }
    
    public function getVersion()
    {
        return $this->moduleList->getOne(self::MODULE_NAME)['setup_version'];
    }
    
    /*
        * @return bool
        */
    public function isEnabled($scope = ScopeInterface::SCOPE_STORE)
    {
        return $this->scopeConfig->isSetFlag(
            'payment/hoolah/active',
            $scope
        );
    }

    /*
        * @return string
        */
    public function getTitle($scope = ScopeInterface::SCOPE_STORE, $store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/title',
            $scope,
            $store
        );
    }
    
    /*
        * @return string
        */
    public function getMerchantCDNID($scope = ScopeInterface::SCOPE_STORE, $store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/hoolah_mc/merchant_cdn_id',
            $scope,
            $store
        );
    }
    
    public function getOrderStatus($scope = ScopeInterface::SCOPE_STORE, $store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/order_status',
            $scope,
            $store
        );
    }
    
    public function getOrderMode($scope = ScopeInterface::SCOPE_STORE, $store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/order_mode',
            $scope,
            $store
        );
    }
    
    public function get_mode($store = null)
    {
        $option = $this->scopeConfig->getValue(
            'payment/hoolah/mode',
            ScopeInterface::SCOPE_STORE,
            $store
        );
        
        $avaliable = [OperationMode::MODE_PRODUCTION, OperationMode::MODE_PRODUCTION_SANDBOX];
        if (HoolahMain::is_dev() && !$this->extSettings->apiHost())
            $avaliable = [OperationMode::MODE_PRODUCTION, OperationMode::MODE_PRODUCTION_SANDBOX, OperationMode::MODE_DEV, OperationMode::MODE_DEV_SANDBOX];
        
        if ($option && in_array($option, $avaliable))
            return $option;
        
        return OperationMode::MODE_PRODUCTION;
    }
    
    public function get_merchant_id($store = null)
    {
        switch ($this->get_mode($store))
        {
            case OperationMode::MODE_DEV:
            case OperationMode::MODE_DEV_SANDBOX:
                return $this->scopeConfig->getValue(
                    'payment/hoolah/hoolah_mc/dev_merchant_id',
                    ScopeInterface::SCOPE_STORE,
                    $store
                );
                break;
        }

        return $this->scopeConfig->getValue(
            'payment/hoolah/hoolah_mc/merchant_id',
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }
    
    public function get_merchant_secret($store = null)
    {
        switch ($this->get_mode($store))
        {
            case OperationMode::MODE_PRODUCTION:
                return $this->scopeConfig->getValue(
                    'payment/hoolah/hoolah_mc/merchant_secret',
                    ScopeInterface::SCOPE_STORE,
                    $store
                );
                break;
            case OperationMode::MODE_PRODUCTION_SANDBOX:
                return $this->scopeConfig->getValue(
                    'payment/hoolah/hoolah_mc/merchant_secret_test_mode',
                    ScopeInterface::SCOPE_STORE,
                    $store
                );
                break;
            case OperationMode::MODE_DEV:
                return $this->scopeConfig->getValue(
                    'payment/hoolah/hoolah_mc/dev_merchant_secret',
                    ScopeInterface::SCOPE_STORE,
                    $store
                );
                break;
            case OperationMode::MODE_DEV_SANDBOX:
                return $this->scopeConfig->getValue(
                    'payment/hoolah/hoolah_mc/dev_merchant_secret_test_mode',
                    ScopeInterface::SCOPE_STORE,
                    $store
                );
                break;
        }
        
        return null;
    }
    
    public function get_hoolah_url($store = null)
    {
        switch ($this->get_mode($store))
        {
            case OperationMode::MODE_PRODUCTION: return $this->extSettings->apiHost(HOOLAH_API_HOST_PROD); break;
            case OperationMode::MODE_PRODUCTION_SANDBOX: return HOOLAH_API_HOST_SANDBOX; break;
            case OperationMode::MODE_DEV: return HOOLAH_URL_DEV; break;
            case OperationMode::MODE_DEV_SANDBOX: return HOOLAH_URL_DEV_SANDBOX; break;
        }
        
        return null;
    }
    
    public function credentials_are_empty($store = null)
    {
        return
            empty($this->get_merchant_id($store)) ||
            empty($this->get_merchant_secret($store)) ||
            empty($this->get_hoolah_url($store));
    }
    
    public function get_hoolah_checkout_url($country = null, $store = null)
    {
        $name = null;
        
        switch ($this->get_mode($store))
        {
            case OperationMode::MODE_PRODUCTION:
                if ($this->extSettings->redirectHost())
                    return $this->extSettings->redirectHost();
                
                $name = 'HOOLAH_CHECKOUT_URL_PROD';
                break;
            case OperationMode::MODE_PRODUCTION_SANDBOX: $name = 'HOOLAH_CHECKOUT_URL_SANDBOX'; break;
            case OperationMode::MODE_DEV: $name = 'HOOLAH_CHECKOUT_URL_DEV'; break;
            case OperationMode::MODE_DEV_SANDBOX: $name = 'HOOLAH_CHECKOUT_URL_DEV_SANDBOX'; break;
        }
        
        if ($name && $country && defined($name.'_'.$country))
            $name = $name.'_'.$country;
        else
            $name = $name.'_SG';
        
        if ($name)
            return constant($name);
        
        return null;
    }
    
    public function get_billing_city_field_title($store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/hoolah_cfm/billing_city_field_title',
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }
    
    public function get_create_email_notification($store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/create_email_notification',
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    public function get_auto_shipment($store = null)
    {
        return $this->scopeConfig->getValue(
            'payment/hoolah/auto_shipment',
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    public function get_external_id_prefix($store = null)
    {
        if (HOOLAH_ENABLE_EXTERNAL_ID_PREFIX)
        {
            $value = $this->scopeConfig->getValue(
                'payment/hoolah/hoolah_cfm/external_id_prefix',
                ScopeInterface::SCOPE_STORE,
                $store
            );
            if ($value)
                return $value.'.';
        }
        
        return '';
    }

    public function getRequest()
    {
        return $this->_request;
    }
}
