/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
/*browser:true*/
/*global define*/
define([
  "mage/storage",
  "Magento_Checkout/js/view/payment/default",
  "mage/url",
  "Magento_Checkout/js/model/error-processor",
  "Magento_Checkout/js/model/full-screen-loader",
  "Magento_Checkout/js/model/quote",
], function (storage, Component, url, errorProcessor, fullScreenLoader, quote) {
  "use strict";

  return Component.extend({
    defaults: {
      redirectAfterPlaceOrder: false,
      template: "Hoolah_Hoolah/payment/form",
    },

    //hoolah_validated : false,

    getCode: function () {
      return "hoolah";
    },

    getPlaceOrderDeferredObject: function () {
      var serviceUrl, payload;
      var self = this;

      payload = {
        guestEmail: quote.guestEmail,
        cartId: quote.getQuoteId(),
        billingAddress: quote.billingAddress(),
        shippingAddress: quote.shippingAddress(),
        customerData: window.checkoutConfig.customerData,
      };

      serviceUrl = url.build("hoolah/gateway/validateorder");

      fullScreenLoader.startLoader();

      var result = storage
        .post(serviceUrl, JSON.stringify(payload))
        .fail(function (response) {
          errorProcessor.process(response, self.messageContainer);
        })
        .done(function (response) {
          self.response = response;
        })
        .always(function () {
          fullScreenLoader.stopLoader();
        });

      return result;
    },

    afterPlaceOrder: function () {
      if (this.response.redirect) {
        fullScreenLoader.startLoader();
        window.location.replace(this.response.redirect);
      } else
        this.messageContainer.addErrorMessage({
          message:
            "Something gone wrong with payment. Please contact us, or make order with another payment gateway.",
        });
    },

    getPaymentAcceptanceMarkSrc: function () {
      return window.checkoutConfig.payment.hoolah.paymentAcceptanceMarkSrc;
    },

    getPaymentAcceptanceIcoSrc: function () {
      return window.checkoutConfig.payment.hoolah.paymentAcceptanceIcoSrc;
    },

    isDisabled: function () {
      return window.checkoutConfig.payment.hoolah.disabled;
    },

    getPaymentDescription: function () {
      return window.checkoutConfig.payment.hoolah.paymentDescription;
    },

    getProcessConsumerLink: function () {
      let billingAddressObject = quote.billingAddress(),
        link = "https://www.shopback.sg/shopback-payments";

      if (billingAddressObject && billingAddressObject["countryId"]) {
        let country = billingAddressObject["countryId"].split(":", 2);

        switch (country[0]) {
          case "SG":
            link = "https://www.shopback.sg/shopback-payments";
            break;
          case "MY":
            link = "https://www.shopback.my/shopback-pay-online";
            break;
          case "TH":
            break;
          case "HK":
            break;
        }
      }

      return link;
    },

    getTermsLink: function () {
      let billingAddressObject = quote.billingAddress(),
        link =
          "https://support.shopback.sg/hc/en-gb/articles/4843519567891-ShopBack-Pay-and-ShopBack-PayLater-Terms-and-Conditions";

      if (billingAddressObject && billingAddressObject["countryId"]) {
        let country = billingAddressObject["countryId"].split(":", 2);

        switch (country[0]) {
          case "SG":
            link =
              "https://support.shopback.sg/hc/en-gb/articles/4843519567891-ShopBack-Pay-and-ShopBack-PayLater-Terms-and-Conditions";
            break;
          case "MY":
            link =
              "https://support.shopback.my/hc/en-us/articles/5171311855891-ShopBack-PayLater-SBPL-Terms-of-Use";
            break;
          case "TH":
            link =
              "https://support.shopback.co.th/hc/th/articles/7970962525331-%E0%B8%99%E0%B9%82%E0%B8%A2%E0%B8%9A%E0%B8%B2%E0%B8%A2%E0%B8%84%E0%B8%A7%E0%B8%B2%E0%B8%A1%E0%B9%80%E0%B8%9B-%E0%B8%99%E0%B8%AA-%E0%B8%A7%E0%B8%99%E0%B8%95%E0%B8%B1%E0%B8%A7-ShopBack-PayLater-SBPL-";
            break;
          case "HK":
            break;
        }
      }

      return link;
    },
  });
});
